# Generated by Django 5.0.10 on 2025-01-28 10:17

import django.db.models.expressions
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbportfolio', '0071_alter_trade_options_alter_trade_order'),
    ]

    operations = [
        migrations.AddField(
            model_name='trade',
            name='diff_shares',
            field=models.GeneratedField(db_persist=True, expression=django.db.models.expressions.CombinedExpression(models.F('shares'), '-', models.F('claimed_shares')), output_field=models.DecimalField(decimal_places=4, max_digits=15)),
        ),
    ]
